import _extends from 'babel-runtime/helpers/extends';
/*  strict */
/* eslint-disable no-use-before-define, prefer-template */

import util from 'util';
import { GraphQLInputObjectType, GraphQLObjectType, isInputType, isAbstractType, GraphQLList, GraphQLNonNull } from '../graphql';

import GenericType from '../type/generic';
import { upperFirst } from './misc';


export function toInputObjectType(typeComposer, opts = {}, cache = new Map()) {
  if (typeComposer.hasInputTypeComposer()) {
    return typeComposer.getInputTypeComposer();
  }

  const schemaComposer = typeComposer.constructor.schemaComposer;
  const prefix = opts.prefix || '';
  const postfix = opts.postfix || 'Input';

  const inputTypeName = `${prefix}${typeComposer.getTypeName()}${postfix}`;
  const type = typeComposer.getType();
  if (cache.has(type)) {
    const itc = cache.get(type);
    return itc;
  }

  const inputTypeComposer = new schemaComposer.InputTypeComposer(new GraphQLInputObjectType({
    name: inputTypeName,
    fields: {}
  }));
  cache.set(typeComposer.getType(), inputTypeComposer);

  const fieldNames = typeComposer.getFieldNames();
  const inputFields = {};
  fieldNames.forEach(fieldName => {
    const fieldOpts = _extends({}, opts, {
      fieldName,
      outputTypeName: typeComposer.getTypeName()
    });
    const fc = typeComposer.getFieldConfig(fieldName);
    if (!isAbstractType(fc.type)) {
      const inputType = convertInputObjectField(fc.type, fieldOpts, cache, schemaComposer);
      inputFields[fieldName] = {
        type: inputType,
        description: fc.description
      };
    }
  });
  inputTypeComposer.addFields(inputFields);

  return inputTypeComposer;
}

export function convertInputObjectField(field, opts, cache, schemaComposer) {
  let fieldType = field;

  const wrappers = [];
  while (fieldType instanceof GraphQLList || fieldType instanceof GraphQLNonNull) {
    wrappers.unshift(fieldType.constructor);
    fieldType = fieldType.ofType;
  }

  if (!isInputType(fieldType)) {
    if (fieldType instanceof GraphQLObjectType) {
      const typeOpts = {
        prefix: `${opts.prefix || ''}${upperFirst(opts.outputTypeName || '')}`,
        postfix: opts.postfix || 'Input'
      };
      const tc = new schemaComposer.TypeComposer(fieldType);
      fieldType = toInputObjectType(tc, typeOpts, cache).getType();
    } else {
      // eslint-disable-next-line
      console.error(`graphql-compose: can not convert field '${opts.outputTypeName || ''}.${opts.fieldName || ''}' to InputType` + '\nIt should be GraphQLObjectType, but got \n' + util.inspect(fieldType, { depth: 2, colors: true }));
      fieldType = GenericType;
    }
  }

  const inputFieldType = wrappers.reduce((type, Wrapper) => new Wrapper(type), fieldType);

  return inputFieldType;
}