'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EnumTypeComposer = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _invariant = require('graphql/jsutils/invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _graphql = require('./graphql');

var _is = require('./utils/is');

var _graphqlVersion = require('./utils/graphqlVersion');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*  strict */
/* eslint-disable no-use-before-define */

var EnumTypeComposer = exports.EnumTypeComposer = function () {
  (0, _createClass3.default)(EnumTypeComposer, null, [{
    key: 'create',
    value: function create(opts) {
      var etc = this.createTemp(opts);
      this.schemaComposer.add(etc);
      return etc;
    }
  }, {
    key: 'createTemp',
    value: function createTemp(opts) {
      if (!this.schemaComposer) {
        throw new Error('Class<EnumTypeComposer> must be created by a SchemaComposer.');
      }

      var ETC = void 0;

      if ((0, _is.isString)(opts)) {
        var typeName = opts;
        var NAME_RX = /^[_a-zA-Z][_a-zA-Z0-9]*$/;
        if (NAME_RX.test(typeName)) {
          ETC = new this.schemaComposer.EnumTypeComposer(new _graphql.GraphQLEnumType({
            name: typeName,
            values: _graphqlVersion.graphqlVersion < 13 ? { _OldGraphqlStubValue_: {} } : {}
          }));
        } else {
          var type = this.schemaComposer.typeMapper.createType(typeName);
          if (!(type instanceof _graphql.GraphQLEnumType)) {
            throw new Error('You should provide correct GraphQLEnumType type definition.' + 'Eg. `enum MyType { KEY1 KEY2 KEY3 }`');
          }
          ETC = new this.schemaComposer.EnumTypeComposer(type);
        }
      } else if (opts instanceof _graphql.GraphQLEnumType) {
        ETC = new this.schemaComposer.EnumTypeComposer(opts);
      } else if ((0, _is.isObject)(opts)) {
        var _type = new _graphql.GraphQLEnumType((0, _extends3.default)({}, opts));
        ETC = new this.schemaComposer.EnumTypeComposer(_type);
      } else {
        throw new Error('You should provide GraphQLEnumTypeConfig or string with enum name or SDL');
      }

      return ETC;
    }
  }]);

  function EnumTypeComposer(gqType) {
    (0, _classCallCheck3.default)(this, EnumTypeComposer);

    if (!this.constructor.schemaComposer) {
      throw new Error('Class<EnumTypeComposer> can only be created by a SchemaComposer.');
    }

    if (!(gqType instanceof _graphql.GraphQLEnumType)) {
      throw new Error('EnumTypeComposer accept only GraphQLEnumType in constructor');
    }
    this.gqType = gqType;
  }

  // -----------------------------------------------
  // Value methods
  // -----------------------------------------------

  (0, _createClass3.default)(EnumTypeComposer, [{
    key: 'hasField',
    value: function hasField(name) {
      var values = this.getFields();
      return !!values[name];
    }
  }, {
    key: '_fixEnumBelowV13',
    value: function _fixEnumBelowV13() {
      if (_graphqlVersion.graphqlVersion < 13) {
        if (!this.gqType._values) {
          // Support for graphql@0.11 and below
          this.gqType._values = defineEnumValues(this.gqType, this.gqType._enumConfig.values);
        }
        this.gqType._values = this.gqType._values.filter(function (o) {
          return o.name !== '_OldGraphqlStubValue_';
        });
      }
    }
  }, {
    key: 'getFields',
    value: function getFields() {
      this._fixEnumBelowV13();
      return this.gqType._getNameLookup();
    }
  }, {
    key: 'getField',
    value: function getField(name) {
      var values = this.getFields();

      if (!values[name]) {
        throw new Error(`Cannot get value '${name}' from enum type '${this.getTypeName()}'. Value with such name does not exist.`);
      }

      return values[name];
    }
  }, {
    key: 'getFieldNames',
    value: function getFieldNames() {
      return Object.keys(this.getFields());
    }

    /**
     * Completely replace all values in GraphQL enum type
     * WARNING: this method rewrite an internal GraphQL instance properties.
     */

  }, {
    key: 'setFields',
    value: function setFields(values) {
      // cleanup isDepricated
      Object.keys(values).forEach(function (key) {
        // $FlowFixMe
        delete values[key].isDeprecated; // eslint-disable-line
      });

      this.gqType._enumConfig.values = values;

      // clear builded fields in type
      delete this.gqType._values;
      delete this.gqType._valueLookup;
      delete this.gqType._nameLookup;

      this._fixEnumBelowV13();

      return this;
    }
  }, {
    key: 'setField',
    value: function setField(name, valueConfig) {
      this.addFields({ [name]: valueConfig });
      return this;
    }

    /**
     * Add new fields or replace existed in a GraphQL type
     */

  }, {
    key: 'addFields',
    value: function addFields(newValues) {
      this.setFields((0, _extends3.default)({}, this.getFields(), newValues));
      return this;
    }
  }, {
    key: 'removeField',
    value: function removeField(nameOrArray) {
      var valueNames = Array.isArray(nameOrArray) ? nameOrArray : [nameOrArray];
      var values = this.getFields();
      valueNames.forEach(function (valueName) {
        return delete values[valueName];
      });
      this.setFields((0, _extends3.default)({}, values));
      return this;
    }
  }, {
    key: 'removeOtherFields',
    value: function removeOtherFields(fieldNameOrArray) {
      var keepFieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      var fields = this.getFields();
      Object.keys(fields).forEach(function (fieldName) {
        if (keepFieldNames.indexOf(fieldName) === -1) {
          delete fields[fieldName];
        }
      });
      this.setFields(fields);
      return this;
    }
  }, {
    key: 'reorderFields',
    value: function reorderFields(names) {
      var orderedFields = {};
      var fields = this.getFields();
      names.forEach(function (name) {
        if (fields[name]) {
          orderedFields[name] = fields[name];
          delete fields[name];
        }
      });
      this.setFields((0, _extends3.default)({}, orderedFields, fields));
      return this;
    }
  }, {
    key: 'extendField',
    value: function extendField(name, partialValueConfig) {
      var prevValueConfig = void 0;
      try {
        prevValueConfig = this.getField(name);
      } catch (e) {
        throw new Error(`Cannot extend value '${name}' from enum '${this.getTypeName()}'. Value does not exist.`);
      }

      var valueConfig = (0, _extends3.default)({}, prevValueConfig, partialValueConfig);
      this.setField(name, valueConfig);
      return this;
    }
  }, {
    key: 'deprecateFields',
    value: function deprecateFields(fields) {
      var _this = this;

      var existedFieldNames = this.getFieldNames();

      if (typeof fields === 'string') {
        if (existedFieldNames.indexOf(fields) === -1) {
          throw new Error(`Cannot deprecate unexisted value '${fields}' from enum '${this.getTypeName()}'`);
        }
        this.extendField(fields, { deprecationReason: 'deprecated' });
      } else if (Array.isArray(fields)) {
        fields.forEach(function (field) {
          if (existedFieldNames.indexOf(field) === -1) {
            throw new Error(`Cannot deprecate unexisted value '${field}' from enum '${_this.getTypeName()}'`);
          }
          _this.extendField(field, { deprecationReason: 'deprecated' });
        });
      } else {
        var fieldMap = fields;
        Object.keys(fieldMap).forEach(function (field) {
          if (existedFieldNames.indexOf(field) === -1) {
            throw new Error(`Cannot deprecate unexisted value '${field}' from enum '${_this.getTypeName()}'`);
          }
          var deprecationReason = fieldMap[field];
          _this.extendField(field, { deprecationReason });
        });
      }

      return this;
    }

    // -----------------------------------------------
    // Type methods
    // -----------------------------------------------

  }, {
    key: 'getType',
    value: function getType() {
      return this.gqType;
    }
  }, {
    key: 'getTypePlural',
    value: function getTypePlural() {
      return new _graphql.GraphQLList(this.gqType);
    }
  }, {
    key: 'getTypeNonNull',
    value: function getTypeNonNull() {
      return new _graphql.GraphQLNonNull(this.gqType);
    }
  }, {
    key: 'getTypeName',
    value: function getTypeName() {
      return this.gqType.name;
    }
  }, {
    key: 'setTypeName',
    value: function setTypeName(name) {
      this.gqType.name = name;
      this.gqType._enumConfig.name = name;
      this.constructor.schemaComposer.add(this);
      return this;
    }
  }, {
    key: 'getDescription',
    value: function getDescription() {
      return this.gqType.description || '';
    }
  }, {
    key: 'setDescription',
    value: function setDescription(description) {
      this.gqType.description = description;
      this.gqType._enumConfig.description = description;
      return this;
    }
  }, {
    key: 'clone',
    value: function clone(newTypeName) {
      if (!newTypeName) {
        throw new Error('You should provide newTypeName:string for EnumTypeComposer.clone()');
      }

      var values = this.getFields();
      var newValues = {};
      Object.keys(values).forEach(function (fieldName) {
        newValues[fieldName] = (0, _extends3.default)({}, values[fieldName]);
        delete newValues[fieldName].isDeprecated;
      });

      var cloned = new this.constructor.schemaComposer.EnumTypeComposer(new _graphql.GraphQLEnumType({
        name: newTypeName,
        values: newValues
      }));

      cloned.setDescription(this.getDescription());

      return cloned;
    }
  }]);
  return EnumTypeComposer;
}();

function isPlainObj(obj) {
  return obj && typeof obj === 'object' && !Array.isArray(obj);
}

function defineEnumValues(type, valueMap /* <T> */
) {
  (0, _invariant2.default)(isPlainObj(valueMap), `${type.name} values must be an object with value names as keys.`);
  return Object.keys(valueMap).map(function (valueName) {
    var value = valueMap[valueName];
    (0, _invariant2.default)(isPlainObj(value), `${type.name}.${valueName} must refer to an object with a "value" key ` + `representing an internal value but got: ${String(value)}.`);
    (0, _invariant2.default)(!value.hasOwnProperty('isDeprecated'), `${type.name}.${valueName} should provide "deprecationReason" instead of "isDeprecated".`);
    return {
      name: valueName,
      description: value.description,
      isDeprecated: Boolean(value.deprecationReason),
      deprecationReason: value.deprecationReason,
      astNode: value.astNode,
      value: value.hasOwnProperty('value') ? value.value : valueName
    };
  });
}