'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.InputTypeComposer = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _graphql = require('./graphql');

var _misc = require('./utils/misc');

var _debug = require('./utils/debug');

var _is = require('./utils/is');

var _configAsThunk = require('./utils/configAsThunk');

var _typeByPath = require('./utils/typeByPath');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*  strict */
/* eslint-disable no-use-before-define */

var InputTypeComposer = exports.InputTypeComposer = function () {
  (0, _createClass3.default)(InputTypeComposer, null, [{
    key: 'create',
    value: function create(opts) {
      var itc = this.createTemp(opts);
      this.schemaComposer.add(itc);
      return itc;
    }
  }, {
    key: 'createTemp',
    value: function createTemp(opts) {
      if (!this.schemaComposer) {
        throw new Error('Class<InputTypeComposer> must be created by a SchemaComposer.');
      }

      var ITC = void 0;

      if ((0, _is.isString)(opts)) {
        var typeName = opts;
        var NAME_RX = /^[_a-zA-Z][_a-zA-Z0-9]*$/;
        if (NAME_RX.test(typeName)) {
          ITC = new this.schemaComposer.InputTypeComposer(new _graphql.GraphQLInputObjectType({
            name: typeName,
            fields: function fields() {
              return {};
            }
          }));
        } else {
          var _type = this.schemaComposer.typeMapper.createType(typeName);
          if (!(_type instanceof _graphql.GraphQLInputObjectType)) {
            throw new Error('You should provide correct GraphQLInputObjectType type definition.' + 'Eg. `input MyInputType { name: String! }`');
          }
          ITC = new this.schemaComposer.InputTypeComposer(_type);
        }
      } else if (opts instanceof _graphql.GraphQLInputObjectType) {
        ITC = new this.schemaComposer.InputTypeComposer(opts);
      } else if ((0, _is.isObject)(opts)) {
        var _type2 = new _graphql.GraphQLInputObjectType({
          name: opts.name,
          description: opts.description,
          fields: function fields() {
            return {};
          }
        });
        ITC = new this.schemaComposer.InputTypeComposer(_type2);

        if ((0, _is.isObject)(opts.fields)) {
          ITC.addFields(opts.fields);
        }
      } else {
        throw new Error('You should provide InputObjectConfig or string with type name to InputTypeComposer.create(opts)');
      }

      return ITC;
    }
  }]);

  function InputTypeComposer(gqType) {
    (0, _classCallCheck3.default)(this, InputTypeComposer);

    if (!this.constructor.schemaComposer) {
      throw new Error('Class<InputTypeComposer> can only be created by a SchemaComposer.');
    }

    if (!(gqType instanceof _graphql.GraphQLInputObjectType)) {
      throw new Error('InputTypeComposer accept only GraphQLInputObjectType in constructor');
    }
    this.gqType = gqType;
  }

  // -----------------------------------------------
  // Field methods
  // -----------------------------------------------

  (0, _createClass3.default)(InputTypeComposer, [{
    key: 'getFields',
    value: function getFields() {
      if (!this.gqType._gqcFields) {
        var _fields = this.gqType._typeConfig.fields;
        this.gqType._gqcFields = (0, _misc.resolveMaybeThunk)(_fields) || {};
      }

      return this.gqType._gqcFields;
    }
  }, {
    key: 'getFieldNames',
    value: function getFieldNames() {
      return Object.keys(this.getFields());
    }
  }, {
    key: 'hasField',
    value: function hasField(fieldName) {
      var fields = this.getFields();
      return !!fields[fieldName];
    }

    /**
     * Completely replace all fields in GraphQL type
     * WARNING: this method rewrite an internal GraphQL instance variable.
     */

  }, {
    key: 'setFields',
    value: function setFields(fields) {
      var _this = this;

      this.gqType._gqcFields = fields;

      this.gqType._typeConfig.fields = function () {
        return (0, _configAsThunk.resolveInputConfigMapAsThunk)(_this.constructor.schemaComposer, fields, _this.getTypeName());
      };
      delete this.gqType._fields; // if schema was builded, delete defineFieldMap
      return this;
    }
  }, {
    key: 'setField',
    value: function setField(fieldName, fieldConfig) {
      this.addFields({ [fieldName]: fieldConfig });
      return this;
    }

    /**
     * Add new fields or replace existed in a GraphQL type
     */

  }, {
    key: 'addFields',
    value: function addFields(newFields) {
      this.setFields((0, _extends3.default)({}, this.getFields(), newFields));
      return this;
    }

    /**
     * Add new fields or replace existed (where field name may have dots)
     */

  }, {
    key: 'addNestedFields',
    value: function addNestedFields(newFields) {
      var _this2 = this;

      Object.keys(newFields).forEach(function (fieldName) {
        var fc = newFields[fieldName];
        var names = fieldName.split('.');
        var name = names.shift();

        if (names.length === 0) {
          // single field
          _this2.setField(name, fc);
        } else {
          // nested field
          var childTC = void 0;
          if (!_this2.hasField(name)) {
            childTC = _this2.constructor.schemaComposer.InputTypeComposer.createTemp(`${_this2.getTypeName()}${(0, _misc.upperFirst)(name)}`);
            _this2.setField(name, childTC);
          } else {
            childTC = _this2.getFieldTC(name);
          }
          childTC.addNestedFields({ [names.join('.')]: fc });
        }
      });

      return this;
    }

    /**
     * Get fieldConfig by name
     */

  }, {
    key: 'getField',
    value: function getField(fieldName) {
      var fields = this.getFields();

      if (!fields[fieldName]) {
        throw new Error(`Cannot get field '${fieldName}' from input type '${this.getTypeName()}'. Field does not exist.`);
      }

      return fields[fieldName];
    }
  }, {
    key: 'removeField',
    value: function removeField(fieldNameOrArray) {
      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      var fields = this.getFields();
      fieldNames.forEach(function (fieldName) {
        return delete fields[fieldName];
      });
      this.setFields(fields);
      return this;
    }
  }, {
    key: 'removeOtherFields',
    value: function removeOtherFields(fieldNameOrArray) {
      var keepFieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      var fields = this.getFields();
      Object.keys(fields).forEach(function (fieldName) {
        if (keepFieldNames.indexOf(fieldName) === -1) {
          delete fields[fieldName];
        }
      });
      this.setFields(fields);
      return this;
    }
  }, {
    key: 'extendField',
    value: function extendField(fieldName, parialFieldConfig) {
      var prevFieldConfig = void 0;
      try {
        prevFieldConfig = this.getFieldConfig(fieldName);
      } catch (e) {
        throw new Error(`Cannot extend field '${fieldName}' from input type '${this.getTypeName()}'. Field does not exist.`);
      }

      this.setField(fieldName, (0, _extends3.default)({}, prevFieldConfig, parialFieldConfig));
      return this;
    }
  }, {
    key: 'reorderFields',
    value: function reorderFields(names) {
      var orderedFields = {};
      var fields = this.getFields();
      names.forEach(function (name) {
        if (fields[name]) {
          orderedFields[name] = fields[name];
          delete fields[name];
        }
      });
      this.setFields((0, _extends3.default)({}, orderedFields, fields));
      return this;
    }
  }, {
    key: 'isFieldNonNull',
    value: function isFieldNonNull(fieldName) {
      return this.getFieldType(fieldName) instanceof _graphql.GraphQLNonNull;
    }

    // alias for isFieldNonNull

  }, {
    key: 'isRequired',
    value: function isRequired(fieldName) {
      return this.isFieldNonNull(fieldName);
    }
  }, {
    key: 'getFieldConfig',
    value: function getFieldConfig(fieldName) {
      var fc = this.getField(fieldName);
      if (!fc) {
        throw new Error(`Type ${this.getTypeName()} does not have field with name '${fieldName}'`);
      }

      return (0, _configAsThunk.resolveInputConfigAsThunk)(this.constructor.schemaComposer, fc, fieldName, this.getTypeName());
    }
  }, {
    key: 'getFieldType',
    value: function getFieldType(fieldName) {
      return this.getFieldConfig(fieldName).type;
    }
  }, {
    key: 'getFieldTC',
    value: function getFieldTC(fieldName) {
      var fieldType = (0, _graphql.getNamedType)(this.getFieldType(fieldName));
      if (!(fieldType instanceof _graphql.GraphQLInputObjectType)) {
        throw new Error(`Cannot get InputTypeComposer for field '${fieldName}' in type ${this.getTypeName()}. ` + `This field should be InputObjectType, but it has type '${fieldType.constructor.name}'`);
      }
      return this.constructor.schemaComposer.InputTypeComposer.createTemp(fieldType);
    }
  }, {
    key: 'makeFieldNonNull',
    value: function makeFieldNonNull(fieldNameOrArray) {
      var _this3 = this;

      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      fieldNames.forEach(function (fieldName) {
        if (_this3.hasField(fieldName)) {
          var fieldType = _this3.getFieldType(fieldName);
          if (!(fieldType instanceof _graphql.GraphQLNonNull)) {
            _this3.extendField(fieldName, { type: new _graphql.GraphQLNonNull(fieldType) });
          }
        }
      });
      return this;
    }

    // alias for makeFieldNonNull

  }, {
    key: 'makeRequired',
    value: function makeRequired(fieldNameOrArray) {
      return this.makeFieldNonNull(fieldNameOrArray);
    }
  }, {
    key: 'makeFieldNullable',
    value: function makeFieldNullable(fieldNameOrArray) {
      var _this4 = this;

      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      fieldNames.forEach(function (fieldName) {
        if (_this4.hasField(fieldName)) {
          var fieldType = _this4.getFieldType(fieldName);
          if (fieldType instanceof _graphql.GraphQLNonNull) {
            _this4.extendField(fieldName, { type: fieldType.ofType });
          }
        }
      });
      return this;
    }
  }, {
    key: 'makeOptional',
    value: function makeOptional(fieldNameOrArray) {
      return this.makeFieldNullable(fieldNameOrArray);
    }

    // -----------------------------------------------
    // Type methods
    // -----------------------------------------------

  }, {
    key: 'getType',
    value: function getType() {
      return this.gqType;
    }
  }, {
    key: 'getTypePlural',
    value: function getTypePlural() {
      return new _graphql.GraphQLList(this.gqType);
    }
  }, {
    key: 'getTypeNonNull',
    value: function getTypeNonNull() {
      return new _graphql.GraphQLNonNull(this.gqType);
    }

    /** @deprecated 5.0.0 */

  }, {
    key: 'getTypeAsRequired',
    value: function getTypeAsRequired() {
      (0, _debug.deprecate)('Use `InputTypeComposer.getTypeNonNull` method instead of `getTypeAsRequired`');
      return this.getTypeNonNull();
    }
  }, {
    key: 'getTypeName',
    value: function getTypeName() {
      return this.gqType.name;
    }
  }, {
    key: 'setTypeName',
    value: function setTypeName(name) {
      this.gqType.name = name;
      this.constructor.schemaComposer.add(this);
      return this;
    }
  }, {
    key: 'getDescription',
    value: function getDescription() {
      return this.gqType.description || '';
    }
  }, {
    key: 'setDescription',
    value: function setDescription(description) {
      this.gqType.description = description;
      return this;
    }
  }, {
    key: 'clone',
    value: function clone(newTypeName) {
      var _this5 = this;

      if (!newTypeName) {
        throw new Error('You should provide new type name for clone() method');
      }

      var newFields = {};
      this.getFieldNames().forEach(function (fieldName) {
        var fc = _this5.getFieldConfig(fieldName);
        newFields[fieldName] = (0, _extends3.default)({}, fc);
      });

      return new this.constructor.schemaComposer.InputTypeComposer(new _graphql.GraphQLInputObjectType({
        name: newTypeName,
        fields: newFields
      }));
    }

    // -----------------------------------------------
    // Misc methods
    // -----------------------------------------------

  }, {
    key: 'get',
    value: function get(path) {
      return (0, _typeByPath.typeByPath)(this, path);
    }
  }]);
  return InputTypeComposer;
}();