'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.InterfaceTypeComposer = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _graphql = require('./graphql');

var _is = require('./utils/is');

var _misc = require('./utils/misc');

var _TypeComposer = require('./TypeComposer');

var _configAsThunk = require('./utils/configAsThunk');

var _typeByPath = require('./utils/typeByPath');

var _typeHelpers = require('./utils/typeHelpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var InterfaceTypeComposer = exports.InterfaceTypeComposer = function () {
  (0, _createClass3.default)(InterfaceTypeComposer, null, [{
    key: 'create',
    value: function create(opts) {
      var iftc = this.createTemp(opts);
      this.schemaComposer.add(iftc);
      return iftc;
    }
  }, {
    key: 'createTemp',
    value: function createTemp(opts) {
      if (!this.schemaComposer) {
        throw new Error('Class<InterfaceTypeComposer> must be created by a SchemaComposer.');
      }

      var IFTC = void 0;

      if ((0, _is.isString)(opts)) {
        var typeName = opts;
        var NAME_RX = /^[_a-zA-Z][_a-zA-Z0-9]*$/;
        if (NAME_RX.test(typeName)) {
          IFTC = new this.schemaComposer.InterfaceTypeComposer(new _graphql.GraphQLInterfaceType({
            name: typeName,
            fields: function fields() {
              return {};
            }
          }));
        } else {
          var type = this.schemaComposer.typeMapper.createType(typeName);
          if (!(type instanceof _graphql.GraphQLInterfaceType)) {
            throw new Error('You should provide correct GraphQLInterfaceType type definition.' + 'Eg. `interface MyType { id: ID!, name: String! }`');
          }
          IFTC = new this.schemaComposer.InterfaceTypeComposer(type);
        }
      } else if (opts instanceof _graphql.GraphQLInterfaceType) {
        IFTC = new this.schemaComposer.InterfaceTypeComposer(opts);
      } else if ((0, _is.isObject)(opts)) {
        var _type = new _graphql.GraphQLInterfaceType((0, _extends3.default)({}, opts));
        IFTC = new this.schemaComposer.InterfaceTypeComposer(_type);
      } else {
        throw new Error('You should provide GraphQLInterfaceTypeConfig or string with enum name or SDL');
      }

      return IFTC;
    }
  }]);

  function InterfaceTypeComposer(gqType) {
    (0, _classCallCheck3.default)(this, InterfaceTypeComposer);

    if (!this.constructor.schemaComposer) {
      throw new Error('Class<InterfaceTypeComposer> can only be created by a SchemaComposer.');
    }

    if (!(gqType instanceof _graphql.GraphQLInterfaceType)) {
      throw new Error('InterfaceTypeComposer accept only GraphQLInterfaceType in constructor');
    }
    this.gqType = gqType;
  }

  // -----------------------------------------------
  // Field methods
  // -----------------------------------------------

  (0, _createClass3.default)(InterfaceTypeComposer, [{
    key: 'hasField',
    value: function hasField(name) {
      var fields = this.getFields();
      return !!fields[name];
    }
  }, {
    key: 'getFields',
    value: function getFields() {
      if (!this.gqType._gqcFields) {
        var _fields = this.gqType._typeConfig.fields;
        this.gqType._gqcFields = (0, _misc.resolveMaybeThunk)(_fields) || {};
      }

      return this.gqType._gqcFields;
    }
  }, {
    key: 'getField',
    value: function getField(name) {
      var values = this.getFields();

      if (!values[name]) {
        throw new Error(`Cannot get field '${name}' from interface type '${this.getTypeName()}'. Field does not exist.`);
      }

      return values[name];
    }
  }, {
    key: 'getFieldNames',
    value: function getFieldNames() {
      return Object.keys(this.getFields());
    }
  }, {
    key: 'setFields',
    value: function setFields(fields) {
      var _this = this;

      this.gqType._gqcFields = fields;

      this.gqType._typeConfig.fields = function () {
        return (0, _configAsThunk.resolveOutputConfigMapAsThunk)(_this.constructor.schemaComposer, fields, _this.getTypeName());
      };
      delete this.gqType._fields; // clear builded fields in type
      return this;
    }
  }, {
    key: 'setField',
    value: function setField(name, fieldConfig) {
      this.addFields({ [name]: fieldConfig });
      return this;
    }

    /**
     * Add new fields or replace existed in a GraphQL type
     */

  }, {
    key: 'addFields',
    value: function addFields(newValues) {
      this.setFields((0, _extends3.default)({}, this.getFields(), newValues));
      return this;
    }
  }, {
    key: 'removeField',
    value: function removeField(nameOrArray) {
      var fieldNames = Array.isArray(nameOrArray) ? nameOrArray : [nameOrArray];
      var values = this.getFields();
      fieldNames.forEach(function (valueName) {
        return delete values[valueName];
      });
      this.setFields((0, _extends3.default)({}, values));
      return this;
    }
  }, {
    key: 'removeOtherFields',
    value: function removeOtherFields(fieldNameOrArray) {
      var keepFieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      var fields = this.getFields();
      Object.keys(fields).forEach(function (fieldName) {
        if (keepFieldNames.indexOf(fieldName) === -1) {
          delete fields[fieldName];
        }
      });
      this.setFields(fields);
      return this;
    }
  }, {
    key: 'reorderFields',
    value: function reorderFields(names) {
      var orderedFields = {};
      var fields = this.getFields();
      names.forEach(function (name) {
        if (fields[name]) {
          orderedFields[name] = fields[name];
          delete fields[name];
        }
      });
      this.setFields((0, _extends3.default)({}, orderedFields, fields));
      return this;
    }
  }, {
    key: 'extendField',
    value: function extendField(fieldName, parialFieldConfig) {
      var prevFieldConfig = void 0;
      try {
        prevFieldConfig = this.getFieldConfig(fieldName);
      } catch (e) {
        throw new Error(`Cannot extend field '${fieldName}' from type '${this.getTypeName()}'. Field does not exist.`);
      }

      this.setField(fieldName, (0, _extends3.default)({}, prevFieldConfig, parialFieldConfig));
      return this;
    }
  }, {
    key: 'isFieldNonNull',
    value: function isFieldNonNull(fieldName) {
      return this.getFieldType(fieldName) instanceof _graphql.GraphQLNonNull;
    }
  }, {
    key: 'getFieldConfig',
    value: function getFieldConfig(fieldName) {
      var fc = this.getField(fieldName);
      if (!fc) {
        throw new Error(`Type ${this.getTypeName()} does not have field with name '${fieldName}'`);
      }

      return (0, _configAsThunk.resolveOutputConfigAsThunk)(this.constructor.schemaComposer, fc, fieldName, this.getTypeName());
    }
  }, {
    key: 'getFieldType',
    value: function getFieldType(fieldName) {
      return this.getFieldConfig(fieldName).type;
    }
  }, {
    key: 'getFieldTC',
    value: function getFieldTC(fieldName) {
      var fieldType = (0, _graphql.getNamedType)(this.getFieldType(fieldName));
      if (!(fieldType instanceof _graphql.GraphQLObjectType)) {
        throw new Error(`Cannot get TypeComposer for field '${fieldName}' in type ${this.getTypeName()}. ` + `This field should be ObjectType, but it has type '${fieldType.constructor.name}'`);
      }
      return this.constructor.schemaComposer.TypeComposer.createTemp(fieldType);
    }
  }, {
    key: 'makeFieldNonNull',
    value: function makeFieldNonNull(fieldNameOrArray) {
      var _this2 = this;

      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      fieldNames.forEach(function (fieldName) {
        if (_this2.hasField(fieldName)) {
          var fieldType = _this2.getFieldType(fieldName);
          if (!(fieldType instanceof _graphql.GraphQLNonNull)) {
            _this2.extendField(fieldName, { type: new _graphql.GraphQLNonNull(fieldType) });
          }
        }
      });
      return this;
    }
  }, {
    key: 'makeFieldNullable',
    value: function makeFieldNullable(fieldNameOrArray) {
      var _this3 = this;

      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      fieldNames.forEach(function (fieldName) {
        if (_this3.hasField(fieldName)) {
          var fieldType = _this3.getFieldType(fieldName);
          if (fieldType instanceof _graphql.GraphQLNonNull) {
            _this3.extendField(fieldName, { type: fieldType.ofType });
          }
        }
      });
      return this;
    }
  }, {
    key: 'deprecateFields',
    value: function deprecateFields(fields) {
      var _this4 = this;

      var existedFieldNames = this.getFieldNames();

      if (typeof fields === 'string') {
        if (existedFieldNames.indexOf(fields) === -1) {
          throw new Error(`Cannot deprecate unexisted field '${fields}' from interface type '${this.getTypeName()}'`);
        }
        this.extendField(fields, { deprecationReason: 'deprecated' });
      } else if (Array.isArray(fields)) {
        fields.forEach(function (field) {
          if (existedFieldNames.indexOf(field) === -1) {
            throw new Error(`Cannot deprecate unexisted field '${field}' from interface type '${_this4.getTypeName()}'`);
          }
          _this4.extendField(field, { deprecationReason: 'deprecated' });
        });
      } else {
        var fieldMap = fields;
        Object.keys(fieldMap).forEach(function (field) {
          if (existedFieldNames.indexOf(field) === -1) {
            throw new Error(`Cannot deprecate unexisted field '${field}' from interface type '${_this4.getTypeName()}'`);
          }
          var deprecationReason = fieldMap[field];
          _this4.extendField(field, { deprecationReason });
        });
      }

      return this;
    }
  }, {
    key: 'getFieldArgs',
    value: function getFieldArgs(fieldName) {
      try {
        var fc = this.getFieldConfig(fieldName);
        return fc.args || {};
      } catch (e) {
        throw new Error(`Cannot get field args. Field '${fieldName}' from type '${this.getTypeName()}' does not exist.`);
      }
    }
  }, {
    key: 'hasFieldArg',
    value: function hasFieldArg(fieldName, argName) {
      var fieldArgs = this.getFieldArgs(fieldName);
      return !!fieldArgs[argName];
    }
  }, {
    key: 'getFieldArg',
    value: function getFieldArg(fieldName, argName) {
      var fieldArgs = this.getFieldArgs(fieldName);

      if (!fieldArgs[argName]) {
        throw new Error(`Cannot get arg '${argName}' from type.field '${this.getTypeName()}.${fieldName}'. Argument does not exist.`);
      }

      return fieldArgs[argName];
    }
  }, {
    key: 'getFieldArgType',
    value: function getFieldArgType(fieldName, argName) {
      var ac = this.getFieldArg(fieldName, argName);
      return ac.type;
    }

    // -----------------------------------------------
    // Type methods
    // -----------------------------------------------

  }, {
    key: 'getType',
    value: function getType() {
      return this.gqType;
    }
  }, {
    key: 'getTypePlural',
    value: function getTypePlural() {
      return new _graphql.GraphQLList(this.gqType);
    }
  }, {
    key: 'getTypeNonNull',
    value: function getTypeNonNull() {
      return new _graphql.GraphQLNonNull(this.gqType);
    }
  }, {
    key: 'getTypeName',
    value: function getTypeName() {
      return this.gqType.name;
    }
  }, {
    key: 'setTypeName',
    value: function setTypeName(name) {
      this.gqType.name = name;
      this.gqType._typeConfig.name = name;
      this.constructor.schemaComposer.add(this);
      return this;
    }
  }, {
    key: 'getDescription',
    value: function getDescription() {
      return this.gqType.description || '';
    }
  }, {
    key: 'setDescription',
    value: function setDescription(description) {
      this.gqType.description = description;
      this.gqType._typeConfig.description = description;
      return this;
    }
  }, {
    key: 'clone',
    value: function clone(newTypeName) {
      var _this5 = this;

      if (!newTypeName) {
        throw new Error('You should provide newTypeName:string for InterfaceTypeComposer.clone()');
      }

      var newFields = {};
      this.getFieldNames().forEach(function (fieldName) {
        var fc = _this5.getFieldConfig(fieldName);
        newFields[fieldName] = (0, _extends3.default)({}, fc);
      });

      var cloned = new this.constructor.schemaComposer.InterfaceTypeComposer(new _graphql.GraphQLInterfaceType({
        name: newTypeName,
        fields: newFields
      }));

      cloned.setDescription(this.getDescription());

      return cloned;
    }

    // -----------------------------------------------
    // ResolveType methods
    // -----------------------------------------------

  }, {
    key: 'hasTypeResolver',
    value: function hasTypeResolver(type) {
      var typeResolversMap = this.getTypeResolvers();
      return typeResolversMap.has(type);
    }
  }, {
    key: 'getTypeResolvers',
    value: function getTypeResolvers() {
      if (!this.gqType._gqcTypeResolvers) {
        this.gqType._gqcTypeResolvers = new Map();
      }
      return this.gqType._gqcTypeResolvers;
    }
  }, {
    key: 'getTypeResolverCheckFn',
    value: function getTypeResolverCheckFn(type) {
      var typeResolversMap = this.getTypeResolvers();

      if (!typeResolversMap.has(type)) {
        throw new Error(`Type resolve function in interface '${this.getTypeName()}' is not defined for type ${(0, _misc.inspect)(type)}.`);
      }

      return typeResolversMap.get(type);
    }
  }, {
    key: 'getTypeResolverNames',
    value: function getTypeResolverNames() {
      var typeResolversMap = this.getTypeResolvers();
      var names = [];
      typeResolversMap.forEach(function (resolveFn, composeType) {
        if (composeType instanceof _TypeComposer.TypeComposer) {
          names.push(composeType.getTypeName());
        } else if (composeType && composeType.name) {
          names.push(composeType.name);
        }
      });
      return names;
    }
  }, {
    key: 'getTypeResolverTypes',
    value: function getTypeResolverTypes() {
      var typeResolversMap = this.getTypeResolvers();
      var types = [];
      typeResolversMap.forEach(function (resolveFn, composeType) {
        types.push((0, _typeHelpers.getGraphQLType)(composeType));
      });
      return types;
    }
  }, {
    key: 'setTypeResolvers',
    value: function setTypeResolvers(typeResolversMap) {
      var _this6 = this;

      this._isTypeResolversValid(typeResolversMap);

      this.gqType._gqcTypeResolvers = typeResolversMap;

      // extract GraphQLObjectType from TypeComposer
      var fastEntries = [];
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = typeResolversMap.entries()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _ref = _step.value;

          var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

          var composeType = _ref2[0];
          var checkFn = _ref2[1];

          fastEntries.push([(0, _typeHelpers.getGraphQLType)(composeType), checkFn]);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      var isAsyncRuntime = this._isTypeResolversAsync(typeResolversMap);
      if (isAsyncRuntime) {
        this.gqType._typeConfig.resolveType = function () {
          var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(value, context, info) {
            var _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, _ref4, _ref5, gqType, _checkFn;

            return _regenerator2.default.wrap(function _callee$(_context) {
              while (1) {
                switch (_context.prev = _context.next) {
                  case 0:
                    _iteratorNormalCompletion2 = true;
                    _didIteratorError2 = false;
                    _iteratorError2 = undefined;
                    _context.prev = 3;
                    _iterator2 = fastEntries[Symbol.iterator]();

                  case 5:
                    if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
                      _context.next = 17;
                      break;
                    }

                    _ref4 = _step2.value;
                    _ref5 = (0, _slicedToArray3.default)(_ref4, 2);
                    gqType = _ref5[0];
                    _checkFn = _ref5[1];
                    _context.next = 12;
                    return _checkFn(value, context, info);

                  case 12:
                    if (!_context.sent) {
                      _context.next = 14;
                      break;
                    }

                    return _context.abrupt('return', gqType);

                  case 14:
                    _iteratorNormalCompletion2 = true;
                    _context.next = 5;
                    break;

                  case 17:
                    _context.next = 23;
                    break;

                  case 19:
                    _context.prev = 19;
                    _context.t0 = _context['catch'](3);
                    _didIteratorError2 = true;
                    _iteratorError2 = _context.t0;

                  case 23:
                    _context.prev = 23;
                    _context.prev = 24;

                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                      _iterator2.return();
                    }

                  case 26:
                    _context.prev = 26;

                    if (!_didIteratorError2) {
                      _context.next = 29;
                      break;
                    }

                    throw _iteratorError2;

                  case 29:
                    return _context.finish(26);

                  case 30:
                    return _context.finish(23);

                  case 31:
                    return _context.abrupt('return', null);

                  case 32:
                  case 'end':
                    return _context.stop();
                }
              }
            }, _callee, _this6, [[3, 19, 23, 31], [24,, 26, 30]]);
          }));

          return function (_x, _x2, _x3) {
            return _ref3.apply(this, arguments);
          };
        }();
      } else {
        this.gqType._typeConfig.resolveType = function (value, context, info) {
          var _iteratorNormalCompletion3 = true;
          var _didIteratorError3 = false;
          var _iteratorError3 = undefined;

          try {
            for (var _iterator3 = fastEntries[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
              var _ref6 = _step3.value;

              var _ref7 = (0, _slicedToArray3.default)(_ref6, 2);

              var gqType = _ref7[0];
              var _checkFn2 = _ref7[1];

              if (_checkFn2(value, context, info)) return gqType;
            }
          } catch (err) {
            _didIteratorError3 = true;
            _iteratorError3 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion3 && _iterator3.return) {
                _iterator3.return();
              }
            } finally {
              if (_didIteratorError3) {
                throw _iteratorError3;
              }
            }
          }

          return null;
        };
      }
      delete this.gqType.resolveType; // clear builded fields in type
      return this;
    }
  }, {
    key: '_isTypeResolversValid',
    value: function _isTypeResolversValid(typeResolversMap) {
      if (!(typeResolversMap instanceof Map)) {
        throw new Error(`For interface ${this.getTypeName()} you should provide Map object for type resolvers.`);
      }

      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = typeResolversMap.entries()[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var _ref8 = _step4.value;

          var _ref9 = (0, _slicedToArray3.default)(_ref8, 2);

          var composeType = _ref9[0];
          var checkFn = _ref9[1];

          // checking composeType
          try {
            var type = (0, _typeHelpers.getGraphQLType)(composeType);
            if (!(type instanceof _graphql.GraphQLObjectType)) throw new Error('Must be GraphQLObjectType');
          } catch (e) {
            throw new Error(`For interface type resolver ${this.getTypeName()} you must provide GraphQLObjectType or TypeComposer, but provided ${(0, _misc.inspect)(composeType)}`);
          }

          // checking checkFn
          if (!(0, _is.isFunction)(checkFn)) {
            throw new Error(`Interface ${this.getTypeName()} has invalid check function for type ${(0, _misc.inspect)(composeType)}`);
          }
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }

      return true;
    }

    // eslint-disable-next-line class-methods-use-this

  }, {
    key: '_isTypeResolversAsync',
    value: function _isTypeResolversAsync(typeResolversMap) {
      var res = false;
      var _iteratorNormalCompletion5 = true;
      var _didIteratorError5 = false;
      var _iteratorError5 = undefined;

      try {
        for (var _iterator5 = typeResolversMap.entries()[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
          var _ref10 = _step5.value;

          var _ref11 = (0, _slicedToArray3.default)(_ref10, 2);

          var checkFn = _ref11[1];

          try {
            var r = checkFn({}, {}, {});
            if (r instanceof Promise) {
              r.catch(function () {});
              res = true;
            }
          } catch (e) {
            // noop
          }
        }
      } catch (err) {
        _didIteratorError5 = true;
        _iteratorError5 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion5 && _iterator5.return) {
            _iterator5.return();
          }
        } finally {
          if (_didIteratorError5) {
            throw _iteratorError5;
          }
        }
      }

      return res;
    }
  }, {
    key: 'addTypeResolver',
    value: function addTypeResolver(type, checkFn) {
      var typeResolversMap = this.getTypeResolvers();
      typeResolversMap.set(type, checkFn);
      this.setTypeResolvers(typeResolversMap);
      return this;
    }
  }, {
    key: 'removeTypeResolver',
    value: function removeTypeResolver(type) {
      var typeResolversMap = this.getTypeResolvers();
      typeResolversMap.delete(type);
      this.setTypeResolvers(typeResolversMap);
      return this;
    }

    // -----------------------------------------------
    // Misc methods
    // -----------------------------------------------

  }, {
    key: 'get',
    value: function get(path) {
      return (0, _typeByPath.typeByPath)(this, path);
    }
  }]);
  return InterfaceTypeComposer;
}(); /*  strict */
/* eslint-disable no-use-before-define */

// import invariant from 'graphql/jsutils/invariant';