'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TypeComposer = undefined;

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _graphql = require('./graphql');

var _InterfaceTypeComposer = require('./InterfaceTypeComposer');

var _Resolver = require('./Resolver');

var _misc = require('./utils/misc');

var _is = require('./utils/is');

var _configAsThunk = require('./utils/configAsThunk');

var _toInputObjectType = require('./utils/toInputObjectType');

var _typeByPath = require('./utils/typeByPath');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import { deprecate } from './utils/debug';


// extended GraphQLFieldConfigMap


// extended GraphQLFieldConfig


// extended GraphQLOutputType


// Compose Args -----------------------------


// RELATION -----------------------------
/*  strict */
/* eslint-disable no-use-before-define */

var TypeComposer = exports.TypeComposer = function () {
  (0, _createClass3.default)(TypeComposer, null, [{
    key: 'create',
    value: function create(opts) {
      var tc = this.createTemp(opts);
      var typeName = tc.getTypeName();
      if (typeName !== 'Query' && typeName !== 'Mutation' && typeName !== 'Subscription') {
        this.schemaComposer.add(tc);
      }
      return tc;
    }
  }, {
    key: 'createTemp',
    value: function createTemp(opts) {
      if (!this.schemaComposer) {
        throw new Error('Class<TypeComposer> must be created by a SchemaComposer.');
      }

      var TC = void 0;

      if ((0, _is.isString)(opts)) {
        var typeName = opts;
        var NAME_RX = /^[_a-zA-Z][_a-zA-Z0-9]*$/;
        if (NAME_RX.test(typeName)) {
          TC = new this.schemaComposer.TypeComposer(new _graphql.GraphQLObjectType({
            name: typeName,
            fields: function fields() {
              return {};
            }
          }));
        } else {
          var _type = this.schemaComposer.typeMapper.createType(typeName);
          if (!(_type instanceof _graphql.GraphQLObjectType)) {
            throw new Error('You should provide correct GraphQLObjectType type definition.' + 'Eg. `type MyType { name: String }`');
          }
          TC = new this.schemaComposer.TypeComposer(_type);
        }
      } else if (opts instanceof _graphql.GraphQLObjectType) {
        TC = new this.schemaComposer.TypeComposer(opts);
      } else if ((0, _is.isObject)(opts)) {
        var _type2 = new _graphql.GraphQLObjectType((0, _extends3.default)({}, opts, {
          fields: function fields() {
            return {};
          }
        }));
        TC = new this.schemaComposer.TypeComposer(_type2);

        if ((0, _is.isObject)(opts.fields)) {
          TC.addFields(opts.fields);
        }
      } else {
        throw new Error('You should provide GraphQLObjectTypeConfig or string with type name to TypeComposer.create(opts)');
      }

      return TC;
    }
  }]);

  function TypeComposer(gqType) {
    (0, _classCallCheck3.default)(this, TypeComposer);

    if (!this.constructor.schemaComposer) {
      throw new Error('Class<TypeComposer> can only be created by a SchemaComposer.');
    }

    if (!(gqType instanceof _graphql.GraphQLObjectType)) {
      throw new Error('TypeComposer accept only GraphQLObjectType in constructor');
    }
    this.gqType = gqType;

    // Alive proper Flow type casting in autosuggestions for class with Generics
    // it's required due using <TContext>
    // and Class<> utility type in SchemaComposer
    /* :: return this; */
  }

  // -----------------------------------------------
  // Field methods
  // -----------------------------------------------

  (0, _createClass3.default)(TypeComposer, [{
    key: 'getFields',
    value: function getFields() {
      if (!this.gqType._gqcFields) {
        var _fields = this.gqType._typeConfig.fields;
        this.gqType._gqcFields = (0, _misc.resolveMaybeThunk)(_fields) || {};
      }

      return this.gqType._gqcFields;
    }
  }, {
    key: 'getFieldNames',
    value: function getFieldNames() {
      return Object.keys(this.getFields());
    }
  }, {
    key: 'setFields',
    value: function setFields(fields) {
      var _this = this;

      this.gqType._gqcFields = fields;

      this.gqType._typeConfig.fields = function () {
        return (0, _configAsThunk.resolveOutputConfigMapAsThunk)(_this.constructor.schemaComposer, fields, _this.getTypeName());
      };
      delete this.gqType._fields; // clear builded fields in type
      return this;
    }
  }, {
    key: 'hasField',
    value: function hasField(fieldName) {
      var fields = this.getFields();
      return !!fields[fieldName];
    }
  }, {
    key: 'setField',
    value: function setField(fieldName, fieldConfig) {
      this.addFields({ [fieldName]: fieldConfig });

      return this;
    }

    /**
     * Add new fields or replace existed in a GraphQL type
     */

  }, {
    key: 'addFields',
    value: function addFields(newFields) {
      this.setFields((0, _extends3.default)({}, this.getFields(), newFields));
      return this;
    }

    /**
     * Add new fields or replace existed (where field name may have dots)
     */

  }, {
    key: 'addNestedFields',
    value: function addNestedFields(newFields) {
      var _this2 = this;

      Object.keys(newFields).forEach(function (fieldName) {
        var fc = newFields[fieldName];
        var names = fieldName.split('.');
        var name = names.shift();

        if (names.length === 0) {
          // single field
          _this2.setField(name, fc);
        } else {
          // nested field
          var childTC = void 0;
          if (!_this2.hasField(name)) {
            childTC = _this2.constructor.schemaComposer.TypeComposer.createTemp(`${_this2.getTypeName()}${(0, _misc.upperFirst)(name)}`);
            _this2.setField(name, {
              type: childTC,
              resolve: function resolve() {
                return {};
              }
            });
          } else {
            childTC = _this2.getFieldTC(name);
          }
          childTC.addNestedFields({ [names.join('.')]: fc });
        }
      });

      return this;
    }

    /**
     * Get fieldConfig by name
     */

  }, {
    key: 'getField',
    value: function getField(fieldName) {
      var fields = this.getFields();

      if (!fields[fieldName]) {
        throw new Error(`Cannot get field '${fieldName}' from type '${this.getTypeName()}'. Field does not exist.`);
      }

      return fields[fieldName];
    }
  }, {
    key: 'removeField',
    value: function removeField(fieldNameOrArray) {
      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      var fields = this.getFields();
      fieldNames.forEach(function (fieldName) {
        return delete fields[fieldName];
      });
      this.setFields((0, _extends3.default)({}, fields));
      return this;
    }
  }, {
    key: 'removeOtherFields',
    value: function removeOtherFields(fieldNameOrArray) {
      var keepFieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      var fields = this.getFields();
      Object.keys(fields).forEach(function (fieldName) {
        if (keepFieldNames.indexOf(fieldName) === -1) {
          delete fields[fieldName];
        }
      });
      this.setFields(fields);
      return this;
    }
  }, {
    key: 'extendField',
    value: function extendField(fieldName, parialFieldConfig) {
      var prevFieldConfig = void 0;
      try {
        prevFieldConfig = this.getFieldConfig(fieldName);
      } catch (e) {
        throw new Error(`Cannot extend field '${fieldName}' from type '${this.getTypeName()}'. Field does not exist.`);
      }

      this.setField(fieldName, (0, _extends3.default)({}, prevFieldConfig, parialFieldConfig));
      return this;
    }
  }, {
    key: 'reorderFields',
    value: function reorderFields(names) {
      var orderedFields = {};
      var fields = this.getFields();
      names.forEach(function (name) {
        if (fields[name]) {
          orderedFields[name] = fields[name];
          delete fields[name];
        }
      });
      this.setFields((0, _extends3.default)({}, orderedFields, fields));
      return this;
    }
  }, {
    key: 'isFieldNonNull',
    value: function isFieldNonNull(fieldName) {
      return this.getFieldType(fieldName) instanceof _graphql.GraphQLNonNull;
    }
  }, {
    key: 'getFieldConfig',
    value: function getFieldConfig(fieldName) {
      var fc = this.getField(fieldName);
      if (!fc) {
        throw new Error(`Type ${this.getTypeName()} does not have field with name '${fieldName}'`);
      }

      return (0, _configAsThunk.resolveOutputConfigAsThunk)(this.constructor.schemaComposer, fc, fieldName, this.getTypeName());
    }
  }, {
    key: 'getFieldType',
    value: function getFieldType(fieldName) {
      return this.getFieldConfig(fieldName).type;
    }
  }, {
    key: 'getFieldTC',
    value: function getFieldTC(fieldName) {
      var fieldType = (0, _graphql.getNamedType)(this.getFieldType(fieldName));
      if (!(fieldType instanceof _graphql.GraphQLObjectType)) {
        throw new Error(`Cannot get TypeComposer for field '${fieldName}' in type ${this.getTypeName()}. ` + `This field should be ObjectType, but it has type '${fieldType.constructor.name}'`);
      }
      return this.constructor.schemaComposer.TypeComposer.createTemp(fieldType);
    }
  }, {
    key: 'makeFieldNonNull',
    value: function makeFieldNonNull(fieldNameOrArray) {
      var _this3 = this;

      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      fieldNames.forEach(function (fieldName) {
        if (_this3.hasField(fieldName)) {
          var fieldType = _this3.getFieldType(fieldName);
          if (!(fieldType instanceof _graphql.GraphQLNonNull)) {
            _this3.extendField(fieldName, { type: new _graphql.GraphQLNonNull(fieldType) });
          }
        }
      });
      return this;
    }
  }, {
    key: 'makeFieldNullable',
    value: function makeFieldNullable(fieldNameOrArray) {
      var _this4 = this;

      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      fieldNames.forEach(function (fieldName) {
        if (_this4.hasField(fieldName)) {
          var fieldType = _this4.getFieldType(fieldName);
          if (fieldType instanceof _graphql.GraphQLNonNull) {
            _this4.extendField(fieldName, { type: fieldType.ofType });
          }
        }
      });
      return this;
    }
  }, {
    key: 'deprecateFields',
    value: function deprecateFields(fields) {
      var _this5 = this;

      var existedFieldNames = this.getFieldNames();

      if (typeof fields === 'string') {
        if (existedFieldNames.indexOf(fields) === -1) {
          throw new Error(`Cannot deprecate unexisted field '${fields}' from type '${this.getTypeName()}'`);
        }
        this.extendField(fields, { deprecationReason: 'deprecated' });
      } else if (Array.isArray(fields)) {
        fields.forEach(function (field) {
          if (existedFieldNames.indexOf(field) === -1) {
            throw new Error(`Cannot deprecate unexisted field '${field}' from type '${_this5.getTypeName()}'`);
          }
          _this5.extendField(field, { deprecationReason: 'deprecated' });
        });
      } else {
        var fieldMap = fields;
        Object.keys(fieldMap).forEach(function (field) {
          if (existedFieldNames.indexOf(field) === -1) {
            throw new Error(`Cannot deprecate unexisted field '${field}' from type '${_this5.getTypeName()}'`);
          }
          var deprecationReason = fieldMap[field];
          _this5.extendField(field, { deprecationReason });
        });
      }

      return this;
    }
  }, {
    key: 'getFieldArgs',
    value: function getFieldArgs(fieldName) {
      try {
        var fc = this.getFieldConfig(fieldName);
        return fc.args || {};
      } catch (e) {
        throw new Error(`Cannot get field args. Field '${fieldName}' from type '${this.getTypeName()}' does not exist.`);
      }
    }
  }, {
    key: 'hasFieldArg',
    value: function hasFieldArg(fieldName, argName) {
      var fieldArgs = this.getFieldArgs(fieldName);
      return !!fieldArgs[argName];
    }
  }, {
    key: 'getFieldArg',
    value: function getFieldArg(fieldName, argName) {
      var fieldArgs = this.getFieldArgs(fieldName);

      if (!fieldArgs[argName]) {
        throw new Error(`Cannot get arg '${argName}' from type.field '${this.getTypeName()}.${fieldName}'. Argument does not exist.`);
      }

      return fieldArgs[argName];
    }
  }, {
    key: 'getFieldArgType',
    value: function getFieldArgType(fieldName, argName) {
      var ac = this.getFieldArg(fieldName, argName);
      return ac.type;
    }

    // -----------------------------------------------
    // Type methods
    // -----------------------------------------------

  }, {
    key: 'getType',
    value: function getType() {
      return this.gqType;
    }
  }, {
    key: 'getTypePlural',
    value: function getTypePlural() {
      return new _graphql.GraphQLList(this.gqType);
    }
  }, {
    key: 'getTypeNonNull',
    value: function getTypeNonNull() {
      return new _graphql.GraphQLNonNull(this.gqType);
    }
  }, {
    key: 'getTypeName',
    value: function getTypeName() {
      return this.gqType.name;
    }
  }, {
    key: 'setTypeName',
    value: function setTypeName(name) {
      this.gqType.name = name;
      this.gqType._typeConfig.name = name;
      this.constructor.schemaComposer.add(this);
      return this;
    }
  }, {
    key: 'getDescription',
    value: function getDescription() {
      return this.gqType.description || '';
    }
  }, {
    key: 'setDescription',
    value: function setDescription(description) {
      this.gqType.description = description;
      this.gqType._typeConfig.description = description;
      return this;
    }
  }, {
    key: 'clone',
    value: function clone(newTypeName) {
      var _this6 = this;

      if (!newTypeName) {
        throw new Error('You should provide newTypeName:string for TypeComposer.clone()');
      }

      var newFields = {};
      this.getFieldNames().forEach(function (fieldName) {
        var fc = _this6.getFieldConfig(fieldName);
        newFields[fieldName] = (0, _extends3.default)({}, fc);
      });

      var cloned = new this.constructor.schemaComposer.TypeComposer(new _graphql.GraphQLObjectType({
        name: newTypeName,
        fields: newFields
      }));

      cloned.setDescription(this.getDescription());
      try {
        cloned.setRecordIdFn(this.getRecordIdFn());
      } catch (e) {
        // no problem, clone without resolveIdFn
      }
      this.getResolvers().forEach(function (resolver) {
        var newResolver = resolver.clone();
        cloned.addResolver(newResolver);
      });

      return cloned;
    }

    // -----------------------------------------------
    // InputType methods
    // -----------------------------------------------

  }, {
    key: 'getInputType',
    value: function getInputType() {
      return this.getInputTypeComposer().getType();
    }
  }, {
    key: 'hasInputTypeComposer',
    value: function hasInputTypeComposer() {
      return !!this.gqType._gqcInputTypeComposer;
    }
  }, {
    key: 'getInputTypeComposer',
    value: function getInputTypeComposer() {
      if (!this.gqType._gqcInputTypeComposer) {
        this.gqType._gqcInputTypeComposer = (0, _toInputObjectType.toInputObjectType)(this);
      }

      return this.gqType._gqcInputTypeComposer;
    }

    // Alias for getInputTypeComposer()

  }, {
    key: 'getITC',
    value: function getITC() {
      return this.getInputTypeComposer();
    }

    // -----------------------------------------------
    // Resolver methods
    // -----------------------------------------------

  }, {
    key: 'getResolvers',
    value: function getResolvers() {
      if (!this.gqType._gqcResolvers) {
        this.gqType._gqcResolvers = new Map();
      }
      return this.gqType._gqcResolvers;
    }
  }, {
    key: 'hasResolver',
    value: function hasResolver(name) {
      if (!this.gqType._gqcResolvers) {
        return false;
      }
      return this.gqType._gqcResolvers.has(name);
    }
  }, {
    key: 'getResolver',
    value: function getResolver(name) {
      if (!this.hasResolver(name)) {
        throw new Error(`Type ${this.getTypeName()} does not have resolver with name '${name}'`);
      }
      var resolverMap = this.gqType._gqcResolvers;
      return resolverMap.get(name);
    }
  }, {
    key: 'setResolver',
    value: function setResolver(name, resolver) {
      if (!this.gqType._gqcResolvers) {
        this.gqType._gqcResolvers = new Map();
      }
      if (!(resolver instanceof _Resolver.Resolver)) {
        throw new Error('setResolver() accept only Resolver instance');
      }
      this.gqType._gqcResolvers.set(name, resolver);
      resolver.setDisplayName(`${this.getTypeName()}.${resolver.name}`);
      return this;
    }
  }, {
    key: 'addResolver',
    value: function addResolver(opts) {
      if (!opts) {
        throw new Error('addResolver called with empty Resolver');
      }

      var resolver = void 0;
      if (!(opts instanceof _Resolver.Resolver)) {
        var resolverOpts = (0, _extends3.default)({}, opts);
        // add resolve method, otherwise added resolver will not return any data by graphql-js
        if (!resolverOpts.hasOwnProperty('resolve')) {
          resolverOpts.resolve = function () {
            return {};
          };
        }
        resolver = new this.constructor.schemaComposer.Resolver(resolverOpts);
      } else {
        resolver = opts;
      }

      if (!resolver.name) {
        throw new Error('resolver should have non-empty `name` property');
      }
      this.setResolver(resolver.name, resolver);
      return this;
    }
  }, {
    key: 'removeResolver',
    value: function removeResolver(resolverName) {
      if (resolverName) {
        this.getResolvers().delete(resolverName);
      }
      return this;
    }
  }, {
    key: 'wrapResolver',
    value: function wrapResolver(resolverName, cbResolver) {
      var resolver = this.getResolver(resolverName);
      var newResolver = resolver.wrap(cbResolver);
      this.setResolver(resolverName, newResolver);
      return this;
    }
  }, {
    key: 'wrapResolverAs',
    value: function wrapResolverAs(resolverName, fromResolverName, cbResolver) {
      var resolver = this.getResolver(fromResolverName);
      var newResolver = resolver.wrap(cbResolver);
      this.setResolver(resolverName, newResolver);
      return this;
    }
  }, {
    key: 'wrapResolverResolve',
    value: function wrapResolverResolve(resolverName, cbNextRp) {
      var resolver = this.getResolver(resolverName);
      this.setResolver(resolverName, resolver.wrapResolve(cbNextRp));
      return this;
    }

    // -----------------------------------------------
    // Interface methods
    // -----------------------------------------------

  }, {
    key: 'getInterfaces',
    value: function getInterfaces() {
      if (!this.gqType._gqcInterfaces) {
        var _interfaces = this.gqType._typeConfig.interfaces;
        this.gqType._gqcInterfaces = (0, _misc.resolveMaybeThunk)(_interfaces) || [];
      }

      return this.gqType._gqcInterfaces;
    }
  }, {
    key: 'setInterfaces',
    value: function setInterfaces(interfaces) {
      var _this7 = this;

      this.gqType._gqcInterfaces = interfaces;
      this.gqType._typeConfig.interfaces = function () {
        return interfaces.map(function (iface) {
          if (iface instanceof _graphql.GraphQLInterfaceType) {
            return iface;
          } else if (iface instanceof _InterfaceTypeComposer.InterfaceTypeComposer) {
            return iface.getType();
          }
          throw new Error(`For type ${_this7.getTypeName()} you provide incorrect interface object ${(0, _misc.inspect)(iface)}`);
        });
      };
      delete this.gqType._interfaces; // if schema was builded, delete _interfaces
      return this;
    }
  }, {
    key: 'hasInterface',
    value: function hasInterface(interfaceObj) {
      return this.getInterfaces().indexOf(interfaceObj) > -1;
    }
  }, {
    key: 'addInterface',
    value: function addInterface(interfaceObj) {
      if (!this.hasInterface(interfaceObj)) {
        this.setInterfaces([].concat((0, _toConsumableArray3.default)(this.getInterfaces()), [interfaceObj]));
      }
      return this;
    }
  }, {
    key: 'removeInterface',
    value: function removeInterface(interfaceObj) {
      var interfaces = this.getInterfaces();
      var idx = interfaces.indexOf(interfaceObj);
      if (idx > -1) {
        interfaces.splice(idx, 1);
        this.setInterfaces(interfaces);
      }
      return this;
    }

    // -----------------------------------------------
    // Misc methods
    // -----------------------------------------------

  }, {
    key: 'addRelation',
    value: function addRelation(fieldName, opts) {
      var _this8 = this;

      if (!this.gqType._gqcRelations) {
        this.gqType._gqcRelations = {};
      }
      this.gqType._gqcRelations[fieldName] = opts;

      if (opts.hasOwnProperty('resolver')) {
        this.setField(fieldName, function () {
          return _this8._relationWithResolverToFC(opts, fieldName);
        });
      } else if (opts.hasOwnProperty('type')) {
        var fc = opts;
        this.setField(fieldName, fc); // was: () => fc
      }

      return this;
    }
  }, {
    key: 'getRelations',
    value: function getRelations() {
      if (!this.gqType._gqcRelations) {
        this.gqType._gqcRelations = {};
      }
      return this.gqType._gqcRelations;
    }
  }, {
    key: '_relationWithResolverToFC',
    value: function _relationWithResolverToFC(opts) {
      var _this9 = this;

      var fieldName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

      var resolver = (0, _is.isFunction)(opts.resolver) ? opts.resolver() : opts.resolver;

      if (!(resolver instanceof _Resolver.Resolver)) {
        throw new Error('You should provide correct Resolver object for relation ' + `${this.getTypeName()}.${fieldName}`);
      }
      if (opts.type) {
        throw new Error('You can not use `resolver` and `type` properties simultaneously for relation ' + `${this.getTypeName()}.${fieldName}`);
      }
      if (opts.resolve) {
        throw new Error('You can not use `resolver` and `resolve` properties simultaneously for relation ' + `${this.getTypeName()}.${fieldName}`);
      }

      var fieldConfig = resolver.getFieldConfig();
      var argsConfig = (0, _extends3.default)({}, fieldConfig.args);
      var argsProto = {};
      var argsRuntime = [];

      // remove args from config, if arg name provided in args
      //    if `argMapVal`
      //       is `undefined`, then keep arg field in config
      //       is `null`, then just remove arg field from config
      //       is `function`, then remove arg field and run it in resolve
      //       is any other value, then put it to args prototype for resolve
      var optsArgs = opts.prepareArgs || {};

      Object.keys(optsArgs).forEach(function (argName) {
        var argMapVal = optsArgs[argName];
        if (argMapVal !== undefined) {
          delete argsConfig[argName];

          if ((0, _is.isFunction)(argMapVal)) {
            argsRuntime.push([argName, argMapVal]);
          } else if (argMapVal !== null) {
            argsProto[argName] = argMapVal;
          }
        }
      });

      // if opts.catchErrors is undefined then set true, otherwise take it value
      var _opts$catchErrors = opts.catchErrors,
          catchErrors = _opts$catchErrors === undefined ? true : _opts$catchErrors;


      var resolve = function resolve(source, args, context, info) {
        var newArgs = (0, _extends3.default)({}, args, argsProto);
        argsRuntime.forEach(function (_ref) {
          var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
              argName = _ref2[0],
              argFn = _ref2[1];

          newArgs[argName] = argFn(source, args, context, info);
        });

        var payload = fieldConfig.resolve ? fieldConfig.resolve(source, newArgs, context, info) : null;
        return catchErrors ? Promise.resolve(payload).catch(function (e) {
          // eslint-disable-next-line
          console.log(`GQC ERROR: relation for ${_this9.getTypeName()}.${fieldName} throws error:`);
          console.log(e); // eslint-disable-line
          return null;
        }) : payload;
      };

      return {
        type: fieldConfig.type,
        description: opts.description,
        deprecationReason: opts.deprecationReason,
        args: argsConfig,
        resolve,
        projection: opts.projection
      };
    }
  }, {
    key: 'setRecordIdFn',
    value: function setRecordIdFn(fn) {
      this.gqType._gqcGetRecordIdFn = fn;
      return this;
    }
  }, {
    key: 'hasRecordIdFn',
    value: function hasRecordIdFn() {
      return !!this.gqType._gqcGetRecordIdFn;
    }
  }, {
    key: 'getRecordIdFn',
    value: function getRecordIdFn() {
      if (!this.gqType._gqcGetRecordIdFn) {
        throw new Error(`Type ${this.getTypeName()} does not have RecordIdFn`);
      }
      return this.gqType._gqcGetRecordIdFn;
    }

    /**
     * Get function that returns record id, from provided object.
     */

  }, {
    key: 'getRecordId',
    value: function getRecordId(source, args, context) {
      return this.getRecordIdFn()(source, args, context);
    }
  }, {
    key: 'get',
    value: function get(path) {
      return (0, _typeByPath.typeByPath)(this, path);
    }
  }]);
  return TypeComposer;
}();