'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TypeMapper = undefined;

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.isOutputType = isOutputType;
exports.isInputType = isInputType;

var _parser = require('graphql/language/parser');

var _language = require('graphql/language');

var _buildASTSchema = require('graphql/utilities/buildASTSchema');

var _keyValMap = require('graphql/jsutils/keyValMap');

var _keyValMap2 = _interopRequireDefault(_keyValMap);

var _invariant = require('graphql/jsutils/invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _find = require('graphql/jsutils/find');

var _find2 = _interopRequireDefault(_find);

var _values = require('graphql/execution/values');

var _graphql = require('./graphql');

var _json = require('./type/json');

var _json2 = _interopRequireDefault(_json);

var _date = require('./type/date');

var _date2 = _interopRequireDefault(_date);

var _buffer = require('./type/buffer');

var _buffer2 = _interopRequireDefault(_buffer);

var _TypeComposer = require('./TypeComposer');

var _InputTypeComposer = require('./InputTypeComposer');

var _EnumTypeComposer = require('./EnumTypeComposer');

var _Resolver = require('./Resolver');

var _TypeStorage = require('./TypeStorage');

var _is = require('./utils/is');

var _default = require('./directive/default');

var _default2 = _interopRequireDefault(_default);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// eg type Name { field: Int }
// eg. Int, Int!, [Int]
// eg. Int, Float
/*  strict */
/* eslint-disable no-use-before-define, class-methods-use-this, no-unused-vars, no-param-reassign */

function isOutputType(type) {
  return type instanceof _graphql.GraphQLScalarType || type instanceof _graphql.GraphQLObjectType || type instanceof _graphql.GraphQLInterfaceType || type instanceof _graphql.GraphQLUnionType || type instanceof _graphql.GraphQLEnumType || type instanceof _graphql.GraphQLNonNull && isOutputType(type.ofType) || type instanceof _graphql.GraphQLList && isOutputType(type.ofType);
}

function isInputType(type) {
  return type instanceof _graphql.GraphQLScalarType || type instanceof _graphql.GraphQLEnumType || type instanceof _graphql.GraphQLInputObjectType || type instanceof _graphql.GraphQLNonNull && isInputType(type.ofType) || type instanceof _graphql.GraphQLList && isInputType(type.ofType);
}

var RegexpOutputTypeDefinition = /type\s[^{]+\{[^}]+\}/im;
var RegexpInputTypeDefinition = /input\s[^{]+\{[^}]+\}/im;
var RegexpEnumTypeDefinition = /enum\s[^{]+\{[^}]+\}/im;

var TypeMapper = exports.TypeMapper = function () {
  function TypeMapper(schemaComposer) {
    (0, _classCallCheck3.default)(this, TypeMapper);
    this.basicScalars = new Map([
    // graphql basic types
    ['String', _graphql.GraphQLString], ['Float', _graphql.GraphQLFloat], ['Int', _graphql.GraphQLInt], ['Boolean', _graphql.GraphQLBoolean], ['ID', _graphql.GraphQLID]]);

    if (!schemaComposer) {
      throw new Error('TypeMapper must have SchemaComposer instance.');
    }
    this.schemaComposer = schemaComposer;
  }

  (0, _createClass3.default)(TypeMapper, [{
    key: 'get',
    value: function get(name) {
      var basicScalar = this.basicScalars.get(name);
      if (basicScalar) return basicScalar;

      if (!this.schemaComposer.has(name)) {
        if (name === 'JSON' || name === 'Json') {
          this.schemaComposer.set(name, _json2.default);
        } else if (name === 'Date') {
          this.schemaComposer.set(name, _date2.default);
        } else if (name === 'Buffer') {
          this.schemaComposer.set(name, _buffer2.default);
        } else {
          return null;
        }
      }

      var schemaType = this.schemaComposer.get(name);
      if ((0, _graphql.isNamedType)(schemaType) || (0, _graphql.isScalarType)(schemaType)) {
        return schemaType;
      }
      return schemaType.getType();
    }
  }, {
    key: 'set',
    value: function set(name, type) {
      this.schemaComposer.set(name, type);
    }
  }, {
    key: 'has',
    value: function has(name) {
      return this.schemaComposer.has(name);
    }
  }, {
    key: 'getWrapped',
    value: function getWrapped(str) {
      var inputTypeAST = (0, _parser.parseType)(str);
      return typeFromAST(inputTypeAST, this.schemaComposer);
    }
  }, {
    key: 'createType',
    value: function createType(str) {
      var existedType = this.get(str);
      if (existedType) return existedType;

      var astDocument = (0, _parser.parse)(str);

      if (!astDocument || astDocument.kind !== 'Document') {
        throw new Error('You should provide correct type syntax. ' + "Eg. createType('type IntRange { min: Int, max: Int }')");
      }

      var types = parseTypes(astDocument, this.schemaComposer);

      var type = types[0];

      if (type) {
        this.set(type.name, type);
        // Also keep type string representation for avoiding duplicates type defs for same strings
        this.set(str, type);
        return type;
      }

      return undefined;
    }
  }, {
    key: 'parseTypesFromString',
    value: function parseTypesFromString(str) {
      var astDocument = (0, _parser.parse)(str);

      if (!astDocument || astDocument.kind !== 'Document') {
        throw new Error('You should provide correct SDL syntax.');
      }

      return this.parseTypesFromAst(astDocument);
    }
  }, {
    key: 'parseTypesFromAst',
    value: function parseTypesFromAst(astDocument) {
      var typeStorage = new _TypeStorage.TypeStorage();

      for (var i = 0; i < astDocument.definitions.length; i++) {
        var def = astDocument.definitions[i];
        var type = makeSchemaDef(def, this.schemaComposer);
        typeStorage.set(type.name, type);
      }
      return typeStorage;
    }
  }, {
    key: 'convertOutputFieldConfig',
    value: function convertOutputFieldConfig(composeFC) {
      var fieldName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      var typeName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';

      (0, _invariant2.default)(composeFC, `You provide empty argument field config for ${typeName}.${fieldName}`);

      var composeType = void 0;
      var copyProps = void 0;
      var copyArgs = void 0;

      if (composeFC instanceof _graphql.GraphQLList || composeFC instanceof _graphql.GraphQLNonNull) {
        return { type: composeFC };
      } else if ((0, _is.isFunction)(composeFC)) {
        return composeFC;
      } else if (composeFC instanceof _Resolver.Resolver) {
        return composeFC.getFieldConfig();
      } else if (composeFC instanceof _TypeComposer.TypeComposer || composeFC instanceof _EnumTypeComposer.EnumTypeComposer) {
        return {
          type: composeFC.getType(),
          description: composeFC.getDescription()
        };
      } else if (Array.isArray(composeFC)) {
        composeType = composeFC;
      } else if (composeFC.type) {
        var _ref = composeFC,
            type = _ref.type,
            args = _ref.args,
            rest = (0, _objectWithoutProperties3.default)(_ref, ['type', 'args']);

        composeType = type;
        copyProps = rest;
        copyArgs = args;
      } else {
        composeType = composeFC;
      }

      var wrapWithList = 0;
      while (Array.isArray(composeType)) {
        if (composeType.length !== 1) {
          throw new Error(`${typeName}.${fieldName} can accept Array exact with one output type definition`);
        }
        wrapWithList += 1;
        composeType = composeType[0];
      }

      if (composeType instanceof _InputTypeComposer.InputTypeComposer) {
        throw new Error(`You cannot provide InputTypeComposer to the field '${typeName}.${fieldName}'. It should be OutputType.`);
      }

      var fieldConfig = {};
      if (typeof composeType === 'string') {
        if (RegexpInputTypeDefinition.test(composeType)) {
          throw new Error(`${typeName}.${fieldName} should be OutputType, but got input type definition '${composeType}'`);
        }

        if (this.schemaComposer.hasInstance(composeType, _TypeComposer.TypeComposer)) {
          fieldConfig.type = this.schemaComposer.getTC(composeType).getType();
        } else {
          var _type = RegexpOutputTypeDefinition.test(composeType) || RegexpEnumTypeDefinition.test(composeType) ? this.createType(composeType) : this.getWrapped(composeType);

          if (!_type) {
            throw new Error(`${typeName}.${fieldName} cannot convert to OutputType the following string: '${composeType}'`);
          }
          fieldConfig.type = _type;
        }
      } else if (composeType instanceof _TypeComposer.TypeComposer || composeType instanceof _EnumTypeComposer.EnumTypeComposer) {
        fieldConfig.type = composeType.getType();
      } else if (composeType instanceof _Resolver.Resolver) {
        fieldConfig.type = composeType.getType();
      } else {
        fieldConfig.type = composeType;
      }

      if (!fieldConfig.type) {
        throw new Error(`${typeName}.${fieldName} must have some 'type'`);
      }

      if (!(0, _is.isFunction)(fieldConfig.type)) {
        if (!isOutputType(fieldConfig.type)) {
          throw new Error(`${typeName}.${fieldName} provided incorrect OutputType: '${JSON.stringify(composeType)}'`);
        }

        if (wrapWithList > 0) {
          for (var i = 0; i < wrapWithList; i++) {
            fieldConfig.type = new _graphql.GraphQLList(fieldConfig.type);
          }
        }
      }

      if (copyArgs) {
        var _args = this.convertArgConfigMap(copyArgs, fieldName, typeName);
        fieldConfig.args = _args;
      }

      if ((0, _is.isObject)(copyProps)) {
        // copy all other props
        for (var prop in copyProps) {
          if (copyProps.hasOwnProperty(prop)) {
            fieldConfig[prop] = copyProps[prop];
          }
        }
      }

      return fieldConfig;
    }
  }, {
    key: 'convertOutputFieldConfigMap',
    value: function convertOutputFieldConfigMap(composeFields) {
      var _this = this;

      var typeName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

      var fields = {};
      Object.keys(composeFields).forEach(function (name) {
        fields[name] = _this.convertOutputFieldConfig(composeFields[name], name, typeName);
      });

      return fields;
    }
  }, {
    key: 'convertArgConfig',
    value: function convertArgConfig(composeAC) {
      var argName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      var fieldName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
      var typeName = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : '';

      (0, _invariant2.default)(composeAC, `You provide empty argument config for ${typeName}.${fieldName}.${argName}`);

      var composeType = void 0;
      var copyProps = void 0;

      if (composeAC instanceof _graphql.GraphQLList || composeAC instanceof _graphql.GraphQLNonNull) {
        return { type: composeAC };
      } else if (composeAC instanceof _InputTypeComposer.InputTypeComposer || composeAC instanceof _EnumTypeComposer.EnumTypeComposer) {
        return {
          type: composeAC.getType(),
          description: composeAC.getDescription()
        };
      } else if (Array.isArray(composeAC)) {
        composeType = composeAC;
      } else if ((0, _is.isFunction)(composeAC)) {
        return composeAC;
      } else if (composeAC.type) {
        var _ref2 = composeAC,
            type = _ref2.type,
            rest = (0, _objectWithoutProperties3.default)(_ref2, ['type']);

        composeType = type;
        copyProps = rest;
      } else {
        composeType = composeAC;
      }

      var wrapWithList = 0;
      while (Array.isArray(composeType)) {
        if (composeType.length !== 1) {
          throw new Error(`${typeName}.${fieldName}@${argName} can accept Array exact with one input type definition`);
        }
        wrapWithList += 1;
        composeType = composeType[0];
      }

      if (composeType instanceof _TypeComposer.TypeComposer) {
        throw new Error(`You cannot provide TypeComposer to the arg '${typeName}.${fieldName}.@${argName}'. It should be InputType.`);
      }

      var argConfig = {};
      if (typeof composeType === 'string') {
        if (RegexpOutputTypeDefinition.test(composeType)) {
          throw new Error(`${typeName}.${fieldName}@${argName} should be InputType, but got output type definition '${composeType}'`);
        }

        if (this.schemaComposer.hasInstance(composeType, _InputTypeComposer.InputTypeComposer)) {
          argConfig.type = this.schemaComposer.getITC(composeType).getType();
        } else {
          var _type2 = RegexpInputTypeDefinition.test(composeType) || RegexpEnumTypeDefinition.test(composeType) ? this.createType(composeType) : this.getWrapped(composeType);

          if (!_type2) {
            throw new Error(`${typeName}.${fieldName}@${argName} cannot convert to InputType the following string: '${composeType}'`);
          }

          argConfig.type = _type2;
        }
      } else if (composeType instanceof _InputTypeComposer.InputTypeComposer || composeType instanceof _EnumTypeComposer.EnumTypeComposer) {
        argConfig.type = composeType.getType();
      } else {
        argConfig.type = composeType;
      }

      if (!argConfig.type) {
        throw new Error(`${typeName}.${fieldName}@${argName} must have some 'type'`);
      }

      if (!(0, _is.isFunction)(argConfig.type)) {
        if (!isInputType(argConfig.type)) {
          throw new Error(`${typeName}.${fieldName}@${argName} provided incorrect InputType: '${JSON.stringify(composeType)}'`);
        }

        if (wrapWithList > 0) {
          for (var i = 0; i < wrapWithList; i++) {
            argConfig.type = new _graphql.GraphQLList(argConfig.type);
          }
        }
      }

      if ((0, _is.isObject)(copyProps)) {
        // copy all other props
        for (var prop in copyProps) {
          if (copyProps.hasOwnProperty(prop)) {
            argConfig[prop] = copyProps[prop];
          }
        }
      }

      return argConfig;
    }
  }, {
    key: 'convertArgConfigMap',
    value: function convertArgConfigMap(composeArgsConfigMap) {
      var _this2 = this;

      var fieldName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      var typeName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';

      var argsConfigMap = {};
      if (composeArgsConfigMap) {
        Object.keys(composeArgsConfigMap).forEach(function (argName) {
          argsConfigMap[argName] = _this2.convertArgConfig(composeArgsConfigMap[argName], argName, fieldName, typeName);
        });
      }

      return argsConfigMap;
    }
  }, {
    key: 'convertInputFieldConfig',
    value: function convertInputFieldConfig(composeIFC) {
      var fieldName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      var typeName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';

      (0, _invariant2.default)(composeIFC, `You provide empty input field config for ${typeName}.${fieldName}`);

      var composeType = void 0;
      var copyProps = void 0;

      if (composeIFC instanceof _graphql.GraphQLList || composeIFC instanceof _graphql.GraphQLNonNull) {
        return { type: composeIFC };
      } else if (composeIFC instanceof _InputTypeComposer.InputTypeComposer || composeIFC instanceof _EnumTypeComposer.EnumTypeComposer) {
        return {
          type: composeIFC.getType(),
          description: composeIFC.getDescription()
        };
      } else if (Array.isArray(composeIFC)) {
        composeType = composeIFC;
      } else if ((0, _is.isFunction)(composeIFC)) {
        return composeIFC;
      } else if (composeIFC.type) {
        var _ref3 = composeIFC,
            type = _ref3.type,
            rest = (0, _objectWithoutProperties3.default)(_ref3, ['type']);

        composeType = composeIFC.type;
        copyProps = rest;
      } else {
        composeType = composeIFC;
      }

      var wrapWithList = 0;
      while (Array.isArray(composeType)) {
        if (composeType.length !== 1) {
          throw new Error(`${typeName}.${fieldName} can accept Array exact with one input type definition`);
        }
        wrapWithList += 1;
        composeType = composeType[0];
      }

      if (composeType instanceof _TypeComposer.TypeComposer) {
        throw new Error(`You cannot provide TypeComposer to the field '${typeName}.${fieldName}'. It should be InputType.`);
      }

      var fieldConfig = {};
      if (typeof composeType === 'string') {
        if (RegexpOutputTypeDefinition.test(composeType)) {
          throw new Error(`${typeName}.${fieldName} should be InputType, but got output type definition '${composeType}'`);
        }

        if (this.schemaComposer.hasInstance(composeType, _InputTypeComposer.InputTypeComposer)) {
          fieldConfig.type = this.schemaComposer.getITC(composeType).getType();
        } else {
          var _type3 = RegexpInputTypeDefinition.test(composeType) || RegexpEnumTypeDefinition.test(composeType) ? this.createType(composeType) : this.getWrapped(composeType);

          if (!_type3) {
            throw new Error(`${typeName}.${fieldName} cannot convert to InputType the following string: '${composeType}'`);
          }

          fieldConfig.type = _type3;
        }
      } else if (composeType instanceof _InputTypeComposer.InputTypeComposer || composeType instanceof _EnumTypeComposer.EnumTypeComposer) {
        fieldConfig.type = composeType.getType();
      } else {
        fieldConfig.type = composeType;
      }

      if (!fieldConfig.type) {
        throw new Error(`${typeName}.${fieldName} must have some 'type'`);
      }

      if (!(0, _is.isFunction)(fieldConfig.type)) {
        if (!isInputType(fieldConfig.type)) {
          throw new Error(`${typeName}.${fieldName} provided incorrect InputType: '${JSON.stringify(composeType)}'`);
        }

        if (wrapWithList > 0) {
          for (var i = 0; i < wrapWithList; i++) {
            fieldConfig.type = new _graphql.GraphQLList(fieldConfig.type);
          }
        }
      }

      if ((0, _is.isObject)(copyProps)) {
        // copy all other props
        for (var prop in copyProps) {
          if (copyProps.hasOwnProperty(prop)) {
            fieldConfig[prop] = copyProps[prop];
          }
        }
      }

      return fieldConfig;
    }
  }, {
    key: 'convertInputFieldConfigMap',
    value: function convertInputFieldConfigMap(composeFields) {
      var _this3 = this;

      var typeName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

      var fields = {};
      Object.keys(composeFields).forEach(function (name) {
        fields[name] = _this3.convertInputFieldConfig(composeFields[name], name, typeName);
      });

      return fields;
    }
  }]);
  return TypeMapper;
}();

// /////////////////////////////////////////////////////////////////////////////
// From GraphQL-js particles
// /////////////////////////////////////////////////////////////////////////////


function parseTypes(astDocument, schema) {
  var types = [];
  for (var i = 0; i < astDocument.definitions.length; i++) {
    var def = astDocument.definitions[i];
    types[i] = makeSchemaDef(def, schema);
  }
  return types;
}

function typeFromAST(inputTypeAST, schema) {
  var innerType = void 0;
  if (inputTypeAST.kind === _language.Kind.LIST_TYPE) {
    innerType = typeFromAST(inputTypeAST.type, schema);
    return innerType && new _graphql.GraphQLList(innerType);
  }
  if (inputTypeAST.kind === _language.Kind.NON_NULL_TYPE) {
    innerType = typeFromAST(inputTypeAST.type, schema);
    return innerType && new _graphql.GraphQLNonNull(innerType);
  }
  (0, _invariant2.default)(inputTypeAST.kind === _language.Kind.NAMED_TYPE, 'Must be a named type.');
  return schema.typeMapper.get(inputTypeAST.name.value);
}

function typeDefNamed(typeName, schema) {
  var type = schema.typeMapper.get(typeName);
  if (type) {
    return type;
  }
  throw new Error(`Cannot find type with name '${typeName}' in TypeMapper.`);
}

function makeSchemaDef(def, schema) {
  if (!def) {
    throw new Error('def must be defined');
  }

  switch (def.kind) {
    case _language.Kind.OBJECT_TYPE_DEFINITION:
      return makeTypeDef(def, schema);
    case _language.Kind.INTERFACE_TYPE_DEFINITION:
      return makeInterfaceDef(def, schema);
    case _language.Kind.ENUM_TYPE_DEFINITION:
      return makeEnumDef(def);
    // case UNION_TYPE_DEFINITION:
    //   return makeUnionDef(def);
    // case SCALAR_TYPE_DEFINITION:
    //   return makeScalarDef(def);
    case _language.Kind.INPUT_OBJECT_TYPE_DEFINITION:
      return makeInputObjectDef(def, schema);
    default:
      throw new Error(`Type kind "${def.kind}" not supported.`);
  }
}

function getInputDefaultValue(value, type) {
  // check getDirectiveValues become avaliable from 0.10.2
  if (Array.isArray(value.directives) && _values.getDirectiveValues) {
    var vars = (0, _values.getDirectiveValues)(_default2.default, value);
    if (vars && vars.hasOwnProperty('value')) return vars.value;
  }
  return (0, _graphql.valueFromAST)(value.defaultValue, type);
}

function makeInputValues(values, schema) {
  if (!values) return {};
  return (0, _keyValMap2.default)(values, function (value) {
    return value.name.value;
  }, function (value) {
    var type = produceInputType(value.type, schema);
    return {
      type,
      description: (0, _buildASTSchema.getDescription)(value),
      defaultValue: getInputDefaultValue(value, type)
    };
  });
}

function makeFieldDefMap(def, schema) {
  if (!def.fields) return {};
  return (0, _keyValMap2.default)(def.fields, function (field) {
    return field.name.value;
  }, function (field) {
    return {
      type: produceOutputType(field.type, schema),
      description: (0, _buildASTSchema.getDescription)(field),
      args: makeInputValues(field.arguments, schema),
      deprecationReason: getDeprecationReason(field.directives)
    };
  });
}

function makeEnumDef(def) {
  var enumType = new _graphql.GraphQLEnumType({
    name: def.name.value,
    description: (0, _buildASTSchema.getDescription)(def),
    values: !def.values ? {} : (0, _keyValMap2.default)(def.values, function (enumValue) {
      return enumValue.name.value;
    }, function (enumValue) {
      return {
        description: (0, _buildASTSchema.getDescription)(enumValue),
        deprecationReason: getDeprecationReason(enumValue.directives)
      };
    })
  });

  return enumType;
}

function makeInputObjectDef(def, schema) {
  return new _graphql.GraphQLInputObjectType({
    name: def.name.value,
    description: (0, _buildASTSchema.getDescription)(def),
    fields: function fields() {
      return makeInputValues(def.fields, schema);
    }
  });
}

function getNamedTypeAST(typeAST) {
  var namedType = typeAST;
  while (namedType.kind === _language.Kind.LIST_TYPE || namedType.kind === _language.Kind.NON_NULL_TYPE) {
    namedType = namedType.type;
  }
  return namedType;
}

function buildWrappedType(innerType, inputTypeAST) {
  if (inputTypeAST.kind === _language.Kind.LIST_TYPE) {
    return new _graphql.GraphQLList(buildWrappedType(innerType, inputTypeAST.type));
  }
  if (inputTypeAST.kind === _language.Kind.NON_NULL_TYPE) {
    var wrappedType = buildWrappedType(innerType, inputTypeAST.type);
    (0, _invariant2.default)(!(wrappedType instanceof _graphql.GraphQLNonNull), 'No nesting nonnull.');
    return new _graphql.GraphQLNonNull(wrappedType);
  }
  return innerType;
}

function produceOutputType(typeAST, schema) {
  var type = produceType(typeAST, schema);
  (0, _invariant2.default)(isOutputType(type), 'Expected Output type.');
  return type;
}

function produceType(typeAST, schema) {
  var typeName = getNamedTypeAST(typeAST).name.value;
  var typeDef = typeDefNamed(typeName, schema);
  return buildWrappedType(typeDef, typeAST);
}

function produceInputType(typeAST, schema) {
  var type = produceType(typeAST, schema);
  (0, _invariant2.default)(isInputType(type), 'Expected Input type.');
  return type;
}

function produceInterfaceType(typeAST, schema) {
  var type = produceType(typeAST, schema);
  (0, _invariant2.default)(type instanceof _graphql.GraphQLInterfaceType, 'Expected Object type.');
  return type;
}

function makeImplementedInterfaces(def, schema) {
  return def.interfaces && def.interfaces.map(function (iface) {
    return produceInterfaceType(iface, schema);
  });
}

function makeTypeDef(def, schema) {
  var typeName = def.name.value;
  return new _graphql.GraphQLObjectType({
    name: typeName,
    description: (0, _buildASTSchema.getDescription)(def),
    fields: function fields() {
      return makeFieldDefMap(def, schema);
    },
    interfaces: function interfaces() {
      return makeImplementedInterfaces(def, schema);
    }
  });
}

function makeInterfaceDef(def, schema) {
  var typeName = def.name.value;
  return new _graphql.GraphQLInterfaceType({
    name: typeName,
    description: (0, _buildASTSchema.getDescription)(def),
    fields: function fields() {
      return makeFieldDefMap(def, schema);
    }
  });
}

function getDeprecationReason(directives) {
  var deprecatedAST = directives && (0, _find2.default)(directives, function (directive) {
    return directive.name.value === _graphql.GraphQLDeprecatedDirective.name;
  });
  if (!deprecatedAST) {
    return;
  }

  var _getArgumentValues = (0, _values.getArgumentValues)(_graphql.GraphQLDeprecatedDirective, deprecatedAST),
      reason = _getArgumentValues.reason;

  return reason; // eslint-disable-line
}