'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.getProjectionFromAST = getProjectionFromAST;
exports.getProjectionFromASTquery = getProjectionFromASTquery;
exports.getFlatProjectionFromAST = getFlatProjectionFromAST;
exports.extendByFieldProjection = extendByFieldProjection;

var _graphql = require('../graphql');

var _deepmerge = require('./deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var FIELD = _graphql.Kind.FIELD,
    FRAGMENT_SPREAD = _graphql.Kind.FRAGMENT_SPREAD,
    INLINE_FRAGMENT = _graphql.Kind.INLINE_FRAGMENT;

// export type ProjectionType = { [fieldName: string]: $Shape<ProjectionNode> | true };
// export type ProjectionNode = { [fieldName: string]: $Shape<ProjectionNode> } | true;
/*  strict */
/* eslint-disable no-param-reassign, no-lonely-if */

function getProjectionFromAST(context, fieldNode) {
  if (!context) {
    return {};
  }

  var queryProjection = getProjectionFromASTquery(context, fieldNode);
  var queryExtProjection = extendByFieldProjection(context.returnType, queryProjection);
  return queryExtProjection;
}

function getProjectionFromASTquery(context, fieldNode) {
  if (!context) {
    return {};
  }

  var selections = void 0; // Array<FieldNode | InlineFragmentNode | FragmentSpreadNode>;
  if (fieldNode) {
    if (fieldNode.selectionSet) {
      selections = fieldNode.selectionSet.selections;
    }
  } else if (Array.isArray(context.fieldNodes)) {
    // get all selectionSets
    selections = context.fieldNodes.reduce(function (result, source) {
      if (source.selectionSet) {
        result.push.apply(result, (0, _toConsumableArray3.default)(source.selectionSet.selections));
      }
      return result;
    }, []);
  }

  var projection = (selections || []).reduce(function (res, ast) {
    switch (ast.kind) {
      case FIELD:
        {
          var value = ast.name.value;

          if (res[value]) {
            res[value] = (0, _deepmerge2.default)(res[value], getProjectionFromASTquery(context, ast) || true);
          } else {
            res[value] = getProjectionFromASTquery(context, ast) || true;
          }
          return res;
        }
      case INLINE_FRAGMENT:
        return (0, _deepmerge2.default)(res, getProjectionFromASTquery(context, ast));
      case FRAGMENT_SPREAD:
        return (0, _deepmerge2.default)(res, getProjectionFromASTquery(context, context.fragments[ast.name.value]));
      default:
        throw new Error('Unsuported query selection');
    }
  }, {});

  return projection;
}

function getFlatProjectionFromAST(context, fieldNodes) {
  var projection = getProjectionFromAST(context, fieldNodes) || {};
  var flatProjection = {};
  Object.keys(projection).forEach(function (key) {
    flatProjection[key] = !!projection[key];
  });
  return flatProjection;
}

// This method traverse fields and extends current projection
// by projection from fields
function extendByFieldProjection(returnType, projection) {
  var type = returnType;

  while (type instanceof _graphql.GraphQLList || type instanceof _graphql.GraphQLNonNull) {
    type = type.ofType;
  }

  if (!(type instanceof _graphql.GraphQLObjectType)) {
    return projection;
  }

  var proj = projection;
  Object.keys(proj).forEach(function (key) {
    var field = type._fields[key];
    if (!field) return;

    if (field.projection) proj = (0, _deepmerge2.default)(proj, field.projection);
    proj[key] = extendByFieldProjection(field.type, proj[key]);
  });

  return proj;
}