'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.toInputObjectType = toInputObjectType;
exports.convertInputObjectField = convertInputObjectField;

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _graphql = require('../graphql');

var _generic = require('../type/generic');

var _generic2 = _interopRequireDefault(_generic);

var _misc = require('./misc');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*  strict */
/* eslint-disable no-use-before-define, prefer-template */

function toInputObjectType(typeComposer) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var cache = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : new Map();

  if (typeComposer.hasInputTypeComposer()) {
    return typeComposer.getInputTypeComposer();
  }

  var schemaComposer = typeComposer.constructor.schemaComposer;
  var prefix = opts.prefix || '';
  var postfix = opts.postfix || 'Input';

  var inputTypeName = `${prefix}${typeComposer.getTypeName()}${postfix}`;
  var type = typeComposer.getType();
  if (cache.has(type)) {
    var itc = cache.get(type);
    return itc;
  }

  var inputTypeComposer = new schemaComposer.InputTypeComposer(new _graphql.GraphQLInputObjectType({
    name: inputTypeName,
    fields: {}
  }));
  cache.set(typeComposer.getType(), inputTypeComposer);

  var fieldNames = typeComposer.getFieldNames();
  var inputFields = {};
  fieldNames.forEach(function (fieldName) {
    var fieldOpts = (0, _extends3.default)({}, opts, {
      fieldName,
      outputTypeName: typeComposer.getTypeName()
    });
    var fc = typeComposer.getFieldConfig(fieldName);
    if (!(0, _graphql.isAbstractType)(fc.type)) {
      var inputType = convertInputObjectField(fc.type, fieldOpts, cache, schemaComposer);
      inputFields[fieldName] = {
        type: inputType,
        description: fc.description
      };
    }
  });
  inputTypeComposer.addFields(inputFields);

  return inputTypeComposer;
}

function convertInputObjectField(field, opts, cache, schemaComposer) {
  var fieldType = field;

  var wrappers = [];
  while (fieldType instanceof _graphql.GraphQLList || fieldType instanceof _graphql.GraphQLNonNull) {
    wrappers.unshift(fieldType.constructor);
    fieldType = fieldType.ofType;
  }

  if (!(0, _graphql.isInputType)(fieldType)) {
    if (fieldType instanceof _graphql.GraphQLObjectType) {
      var typeOpts = {
        prefix: `${opts.prefix || ''}${(0, _misc.upperFirst)(opts.outputTypeName || '')}`,
        postfix: opts.postfix || 'Input'
      };
      var tc = new schemaComposer.TypeComposer(fieldType);
      fieldType = toInputObjectType(tc, typeOpts, cache).getType();
    } else {
      // eslint-disable-next-line
      console.error(`graphql-compose: can not convert field '${opts.outputTypeName || ''}.${opts.fieldName || ''}' to InputType` + '\nIt should be GraphQLObjectType, but got \n' + _util2.default.inspect(fieldType, { depth: 2, colors: true }));
      fieldType = _generic2.default;
    }
  }

  var inputFieldType = wrappers.reduce(function (type, Wrapper) {
    return new Wrapper(type);
  }, fieldType);

  return inputFieldType;
}